@@ -1,12 +1,11 @@
-# -*- coding: utf-8 -*-
 
-# Copyright 2009-2017 Jaap Karssenberg <jaap.karssenberg@gmail.com>
-
-from __future__ import with_statement
+# Copyright 2009-2018 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 
 
 from datetime import datetime
+from typing import Generator, Optional
 
+import sqlite3
 import logging
 
 logger = logging.getLogger('zim.notebook.index')
@@ -14,8 +13,6 @@ logger = logging.getLogger('zim.notebook.index')
 from zim.utils import natural_sort_key
 from zim.notebook.page import Path, HRef, \
 	HREF_REL_ABSOLUTE, HREF_REL_FLOATING, HREF_REL_RELATIVE
-from zim.tokenparser import TokenBuilder
-
 from zim.formats import ParseTreeBuilder
 
 from .base import *
@@ -46,7 +43,8 @@ class PagesIndexer(IndexerBase):
 
 	@signal: C{page-row-inserted (row)}: new row inserted
 	@signal: C{page-row-changed (row, oldrow)}: row changed
-	@signal: C{page-row-deleted (row)}: row to be deleted
+	@signal: C{page-row-delete (row)}: row to be deleted
+	@signal: C{page-row-deleted (row)}: row that has been deleted
 
 	@signal: C{page-changed (row, content)}: page contents changed
 	'''
@@ -54,6 +52,7 @@ class PagesIndexer(IndexerBase):
 	__signals__ = {
 		'page-row-inserted': (None, None, (object,)),
 		'page-row-changed': (None, None, (object, object)),
+		'page-row-delete': (None, None, (object,)),
 		'page-row-deleted': (None, None, (object,)),
 		'page-changed': (None, None, (object, object))
 	}
@@ -78,6 +77,8 @@ class PagesIndexer(IndexerBase):
 
 				source_file INTEGER REFERENCES files(id),
 				is_link_placeholder BOOLEAN DEFAULT 0
+
+				CONSTRAINT no_self_ref CHECK (parent <> id)
 			);
 			CREATE UNIQUE INDEX IF NOT EXISTS pages_name ON pages(name);
 			CREATE INDEX IF NOT EXISTS pages_sortkey ON pages(sortkey);
@@ -129,7 +130,9 @@ class PagesIndexer(IndexerBase):
 			return # nothing to do
 
 		row = self._select(pagename)
-		assert row is not None
+		if row is None:
+			self.insert_page(pagename, filerow['id']) # May have been changed file type
+			row = self._select(pagename)
 
 		if row['source_file'] == filerow['id']:
 			file = self.layout.root.file(filerow['path'])
@@ -146,7 +149,8 @@ class PagesIndexer(IndexerBase):
 			return # nothing to do
 
 		row = self._select(pagename)
-		assert row is not None
+		if row is None:
+			return # not a source file after all (e.g. .txt attachment)
 
 		if row['source_file'] == filerow['id']:
 			if row['n_children'] > 0:
@@ -164,7 +168,7 @@ class PagesIndexer(IndexerBase):
 			else:
 				self.remove_page(pagename)
 		else:
-			raise NotImplemented # some conflict removed
+			pass # some conflict removed - or not a source file in the first place (.txt attachment)
 
 	def insert_page(self, pagename, file_id):
 		return self._insert_page(pagename, False, file_id)
@@ -191,19 +195,32 @@ class PagesIndexer(IndexerBase):
 			parent_row = self._select(pagename.parent)
 			assert parent_row is not None
 
-		# update table
+		# insert new page
 		lowerbasename = pagename.basename.lower()
 		sortkey = natural_sort_key(pagename.basename)
-		self.db.execute(
-			'INSERT INTO pages(name, lowerbasename, sortkey, parent, is_link_placeholder, source_file)'
-			'VALUES (?, ?, ?, ?, ?, ?)',
-			(pagename.name, lowerbasename, sortkey, parent_row['id'], is_link_placeholder, file_id)
-		)
-		self.update_parent(pagename.parent)
+		try:
+			self.db.execute(
+				'INSERT INTO pages(name, lowerbasename, sortkey, parent, is_link_placeholder, source_file)'
+				'VALUES (?, ?, ?, ?, ?, ?)',
+				(pagename.name, lowerbasename, sortkey, parent_row['id'], is_link_placeholder, file_id)
+			)
+		except sqlite3.IntegrityError:
+			# This can occur in rare edge cases when resolve_page failed to
+			# see a page existed already - typically due to locale changes
+			# affecting sortkey
+			logger.exception('Error while inserting page - re-index needed?')
+			self.db.execute(
+				'UPDATE pages SET sortkey=? WHERE name=?',
+				(sortkey, pagename.name)
+			)
+			row = self._select(pagename)
+		else:
+			row = self._select(pagename)
+			self._update_parent_nchildren(pagename.parent)
+			self.emit('page-row-inserted', row)
 
-		# notify others
-		row = self._select(pagename)
-		self.emit('page-row-inserted', row)
+		# update parent(s)
+		self.update_parent(pagename.parent)
 
 		return row['id']
 
@@ -257,13 +274,33 @@ class PagesIndexer(IndexerBase):
 		# allow_cleanup is used by LinksIndexer when cleaning up placeholders
 
 		row = self._select(pagename)
+		assert row['id'] != 1, 'BUG: can\'t delete notebook root'
 		if row['n_children'] > 0:
 			raise AssertionError('Page has child pages')
 
-		self.emit('page-row-deleted', row)
+		self.emit('page-row-delete', row)
 		self.db.execute('DELETE FROM pages WHERE name=?', (pagename.name,))
+		self._update_parent_nchildren(pagename.parent)
+		self.emit('page-row-deleted', row)
 		self.update_parent(pagename.parent, allow_cleanup)
 
+	def _update_parent_nchildren(self, parentname):
+		# parent n_children needs to be up-to-date when we emit the "deleted"
+		# signal, else Gtk.TreeView sees an inconsistency
+		# We still call update_parent() after the fact to do the rest of the
+		# house keeping
+		row = self._select(parentname)
+		assert row is not None
+
+		n_children, = self.db.execute(
+			'SELECT count(*) FROM pages WHERE parent=?',
+			(row['id'],)
+		).fetchone()
+		self.db.execute(
+			'UPDATE pages SET n_children=? WHERE id=?',
+			(n_children, row['id'])
+		)
+
 
 class PageIndexRecord(Path):
 	'''Object representing a page L{Path} in the index, with data
@@ -316,7 +353,7 @@ class PagesViewInternal(object):
 			raise IndexConsistencyError('No page for page_id "%r"' % page_id)
 		return PageIndexRecord(row)
 
-	def get_page_id(self, pagename):
+	def get_page_id(self, pagename: Path) -> int:
 		row = self.db.execute(
 			'SELECT id FROM pages WHERE name=?', (pagename.name,)
 		).fetchone()
@@ -338,7 +375,7 @@ class PagesViewInternal(object):
 			try:
 				start_id = self.get_page_id(start)
 			except IndexNotFoundError:
-				relnames.append(start.basename)
+				relnames.insert(0, start.basename)
 				start = start.parent
 			else:
 				break
@@ -351,14 +388,19 @@ class PagesViewInternal(object):
 			# By default ignore link placeholders to avoid circular
 			# dependencies between links and placeholders
 			assert href.rel == HREF_REL_FLOATING
+
+			# link to anchor on current page
+			if not href.parts():
+				return (start, start_id, relnames + href.parts())
+
 			anchor_key = natural_sort_key(href.parts()[0])
 
 			if relnames:
 				# Check if we are anchored in non-existing part
-				keys = map(natural_sort_key, relnames)
+				keys = list(map(natural_sort_key, relnames))
 				if anchor_key in keys:
 					i = [c for c, k in enumerate(keys) if k == anchor_key][-1]
-					return (start, start_id, relnames[:i] + href.parts()[1:])
+					return (start, start_id, relnames[:i] + href.parts())
 
 			if ignore_link_placeholders:
 				c = self.db.execute(
@@ -411,18 +453,6 @@ class PagesViewInternal(object):
 					return (start.parent, None, href.parts())
 
 	def resolve_pagename(self, parent, names, parent_id=None):
-		page_id, pagename, branch = self._resolve_pagename(parent, names, parent_id)
-		if page_id is None:
-			try:
-				page_id = self.get_page_id(pagename)
-			except IndexNotFoundError:
-				pass
-			else:
-				logger.error('BUG: issue #19 seen: (%r, %r) -> (None, %r) - branch %i !', parent, names, pagename, branch)
-
-		return page_id, pagename
-
-	def _resolve_pagename(self, parent, names, parent_id=None):
 		'''Resolve a pagename in the right case'''
 		# We do not ignore placeholders here. This can lead to a dependencies
 		# in how links are resolved based on order of indexing. However, this
@@ -441,7 +471,7 @@ class PagesViewInternal(object):
 				(page_id, sortkey)
 			).fetchall()
 
-			exact = pagename + ':' + basename
+			exact = pagename.child(basename).name
 			for row in candidates:
 				if row['name'] == exact:
 					pagename = Path(row['name'])
@@ -453,9 +483,9 @@ class PagesViewInternal(object):
 					pagename = Path(row['name'])
 					page_id = row['id']
 				else: # no match
-					return None, pagename.child(':'.join(names[i:])), 1
+					return None, pagename.child(':'.join(names[i:]))
 		else:
-			return page_id, pagename, 2
+			return page_id, pagename
 
 	def walk(self, parent_id):
 		# Need to do this recursive to preserve sorting
@@ -485,11 +515,11 @@ class PagesViewInternal(object):
 class PagesView(IndexView):
 	'''Index view that exposes the "pages" table in the index'''
 
-	def __init__(self, db):
+	def __init__(self, db: sqlite3.Connection):
 		IndexView.__init__(self, db)
 		self._pages = PagesViewInternal(db)
 
-	def lookup_by_pagename(self, pagename):
+	def lookup_by_pagename(self, pagename: Path) -> PageIndexRecord:
 		r = self.db.execute(
 			'SELECT * FROM pages WHERE name=?', (pagename.name,)
 		).fetchone()
@@ -498,9 +528,8 @@ class PagesView(IndexView):
 		else:
 			return PageIndexRecord(r)
 
-	def list_pages(self, path=None):
+	def list_pages(self, path: Optional[Path] = None) -> Generator[PageIndexRecord, None, None]:
 		'''Generator for child pages of C{path}
-		@param path: a L{Path} object
 		@returns: yields L{Path} objects for children of C{path}
 		@raises IndexNotFoundError: if C{path} is not found in the index
 		'''
@@ -510,24 +539,25 @@ class PagesView(IndexView):
 			page_id = self._pages.get_page_id(path) # can raise
 		return self._list_pages(page_id)
 
-	def _list_pages(self, page_id):
+	def _list_pages(self, page_id: int):
 		for row in self.db.execute(
 			'SELECT * FROM pages WHERE parent=? ORDER BY sortkey, name',
 			(page_id,)
 		):
 			yield PageIndexRecord(row)
 
-	def n_list_pages(self, path=None):
+	def n_list_pages(self, path: Optional[Path] = None) -> int:
+		'''@returns: number of child pages of C{path}.
+		@param path: optional, defaults to root path
+		'''
 		page_id = self._pages.get_page_id(path or ROOT_PATH)
 		c, = self.db.execute(
 			'SELECT COUNT(*) FROM pages WHERE parent=?', (page_id,)
 		).fetchone()
 		return c
 
-	def match_pages(self, path, text, limit=10):
+	def match_pages(self, path: Path, text: str, limit: int = 10) -> Generator[PageIndexRecord, None, None]:
 		'''Generator for child pages of C{path} that match C{text} in their name
-		@param path: a L{Path} object
-		@param text: a string
 		@param limit: max number of results
 		@returns: yields L{Path} objects for children of C{path}
 		@raises IndexNotFoundError: if C{path} is not found in the index
@@ -538,7 +568,7 @@ class PagesView(IndexView):
 			page_id = self._pages.get_page_id(path) # can raise
 		return self._match_pages(page_id, text, limit)
 
-	def _match_pages(self, page_id, text, limit):
+	def _match_pages(self, page_id: int, text: str, limit: int):
 		# The LIKE keyword does not handle unicode case-insensitivity
 		# therefore we need python lower() to do the job
 		for row in self.db.execute(
@@ -547,7 +577,7 @@ class PagesView(IndexView):
 		):
 			yield PageIndexRecord(row)
 
-	def match_all_pages(self, text, limit=10):
+	def match_all_pages(self, text: str, limit: int = 10) -> Generator[PageIndexRecord, None, None]:
 		'''Like C{match_pages()} except not limited a specific namespace'''
 		for row in self.db.execute(
 			'SELECT * FROM pages WHERE lowerbasename LIKE ? ORDER BY length(name), sortkey, name LIMIT ?',
@@ -555,7 +585,24 @@ class PagesView(IndexView):
 		):
 			yield PageIndexRecord(row)
 
-	def walk(self, path=None):
+
+	def match_all_pages_by_words(self, words: list, limit: int = 10) -> Generator[PageIndexRecord, None, None]:
+		'''Like C{match_all_pages()}, except it performs a search based on multiple words'''
+
+		query_fragments = ['SELECT * FROM pages WHERE 1']
+		query_parameters = []
+
+		for w in words:
+			query_fragments.append("AND name LIKE ?")
+			query_parameters.append("%%%s%%" % w.lower())
+
+		query_fragments.append('ORDER BY length(name), sortkey, name LIMIT ?')
+		query_parameters.append(limit)
+
+		for row in self.db.execute(" ".join(query_fragments), query_parameters):
+			yield PageIndexRecord(row)
+
+	def walk(self, path: Optional[Path] = None) -> Generator[PageIndexRecord, None, None]:
 		'''Generator function to yield all pages in the index, depth
 		first
 
@@ -570,19 +617,38 @@ class PagesView(IndexView):
 		page_id = self._pages.get_page_id(path) if path else ROOT_ID # can raise
 		return self._pages.walk(page_id)
 
-	def walk_bottomup(self, path=None):
+	def walk_bottomup(self, path: Optional[Path] = None) -> Generator[PageIndexRecord, None, None]:
 		page_id = self._pages.get_page_id(path) if path else ROOT_ID # can raise
 		return self._pages.walk_bottomup(page_id)
 
-	def n_all_pages(self):
-		'''Returns to total number of pages in the index'''
+	def n_all_pages(self) -> int:
+		'''@returns: total number of pages in the index'''
 		c, = self.db.execute('SELECT COUNT(*) FROM pages').fetchone()
 		return c - 1 # don't count ROOT
 
-	def get_previous(self, path):
+	def get_has_previous_has_next(self, path: Path) -> bool:
+		if path.isroot:
+			raise ValueError('Can\'t use root')
+
+		r = self.db.execute(
+			'SELECT * FROM pages WHERE parent=? '
+			'ORDER BY sortkey ASC, name ASC LIMIT 1',
+			(ROOT_ID,)
+		).fetchone()
+		is_first = (r['name'] == path.name) if r else True
+
+		r = self.db.execute(
+			'SELECT * FROM pages WHERE parent=? '
+			'ORDER BY sortkey DESC, name DESC LIMIT 1',
+			(ROOT_ID,)
+		).fetchone()
+		is_last = (r['name'] == path.name) if r else True
+
+		return not is_first, not is_last
+
+	def get_previous(self, path: Path) -> Optional[Path]:
 		'''Get the previous path in the index, in the same order that
 		L{walk()} will yield them
-		@param path: a L{Path} object
 		@returns: a L{Path} object or C{None} if {path} is the first page in
 		the index
 		'''
@@ -621,10 +687,9 @@ class PagesView(IndexView):
 			else:
 				return PageIndexRecord(r)
 
-	def get_next(self, path):
+	def get_next(self, path: Path) -> Optional[Path]:
 		'''Get the next path in the index, in the same order that
 		L{walk()} will yield them
-		@param path: a L{Path} object
 		@returns: a L{Path} object or C{None} if C{path} is the last page in
 		the index
 		'''
@@ -669,7 +734,7 @@ class PagesView(IndexView):
 					if r is None:
 						raise IndexConsistencyError('Missing parent')
 
-	def lookup_from_user_input(self, name, reference=None):
+	def lookup_from_user_input(self, name: str, reference: Path = None) -> Path:
 		'''Lookup a pagename based on user input
 		@param name: the user input as string
 		@param reference: a L{Path} in case relative links are supported as
@@ -692,7 +757,7 @@ class PagesView(IndexView):
 								source, href, ignore_link_placeholders=False)
 			return pagename
 
-	def resolve_link(self, source, href):
+	def resolve_link(self, source: Path, href: HRef) -> Path:
 		'''Find the end point of a link
 		Depending on the link type (absolute, relative, or floating),
 		this method first determines the starting point of the link
@@ -707,11 +772,8 @@ class PagesView(IndexView):
 		id, pagename = self._pages.resolve_link(source, href)
 		return pagename
 
-	def create_link(self, source, target):
+	def create_link(self, source: Path, target: Path) -> HRef:
 		'''Determine best way to represent a link between two pages
-		@param source: a L{Path} object
-		@param target: a L{Path} object
-		@returns: a L{HRef} object
 		'''
 		if target == source: # weird edge case ..
 			return HRef(HREF_REL_FLOATING, target.basename)
@@ -721,7 +783,7 @@ class PagesView(IndexView):
 			href = self._find_floating_link(source, target)
 			return href or HRef(HREF_REL_ABSOLUTE, target.name)
 
-	def _find_floating_link(self, source, target):
+	def _find_floating_link(self, source: Path, target: Path) -> Optional[Path]:
 		# Relative links only resolve for pages that have a common parent
 		# with the source page. So we start finding the common parents and
 		# if that does not resolve (e.g. because same name also occurs on a
@@ -756,7 +818,7 @@ class PagesView(IndexView):
 			else:
 				return None # no floating link possible
 
-	def list_recent_changes(self, limit=None, offset=None):
+	def list_recent_changes(self, limit: Optional[int] = None, offset: Optional[int] = None) -> Generator[PageIndexRecord, None, None]:
 		assert not (offset and not limit), "Can't use offset without limit"
 		if limit:
 			selection = ' LIMIT %i OFFSET %i' % (limit, offset or 0)
@@ -769,6 +831,12 @@ class PagesView(IndexView):
 			yield PageIndexRecord(row)
 
 
+try:
+	from gi.repository import Gtk
+except ImportError:
+	Gtk = None
+
+
 IS_PAGE = 1 #: Hint for MyTreeIter
 
 class PagesTreeModelMixin(TreeModelMixinBase):
@@ -780,22 +848,45 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 	# Signals use "find_all" instead of "find" to allow for subclasses that
 	# have multiple entries, like models for tags
 
+	def __init__(self, index, root=None, reverse=False):
+		TreeModelMixinBase.__init__(self, index)
+		self._REVERSE = reverse
+		if root is None:
+			self._MY_ROOT_NAME = ''
+			self._MY_ROOT_NAME_C = ''
+			self._MY_ROOT_ID = ROOT_ID
+		else:
+			self._MY_ROOT_NAME = root.name
+			self._MY_ROOT_NAME_C = root.name + ':'
+			self._set_root_id()
+
+		self._deleted_paths = None
+
+	def _set_root_id(self):
+		myrow = self.db.execute(
+			'SELECT * FROM pages WHERE name=?', (self._MY_ROOT_NAME,)
+		).fetchone()
+		self._MY_ROOT_ID = myrow['id'] if myrow else None
+
 	def connect_to_updateiter(self, index, update_iter):
 		self.connectto_all(update_iter.pages,
-			('page-row-inserted', 'page-row-changed', 'page-row-deleted')
+			('page-row-inserted', 'page-row-changed', 'page-row-delete', 'page-row-deleted')
 		)
 
 	def on_page_row_inserted(self, o, row):
 		self.flush_cache()
-		for treepath in self._find_all_pages(row['name']):
-			if treepath[-1] == 0 and len(treepath) > 1:
-				self._check_parent_has_child_toggled(treepath)
-			treeiter = self.get_iter(treepath) # not mytreeiter !
-			self.emit('row-inserted', treepath, treeiter)
+		if row['name'] == self._MY_ROOT_NAME:
+			self._set_root_id()
+		else:
+			for treepath in self._find_all_pages(row['name']):
+				treeiter = self.get_iter(treepath) # not mytreeiter !
+				self.emit('row-inserted', treepath, treeiter)
+				if treepath[-1] == 0 and len(treepath) > 1:
+					self._check_parent_has_child_toggled(treepath, 1)
 
-	def _check_parent_has_child_toggled(self, treepath):
+	def _check_parent_has_child_toggled(self, treepath, count):
 		parent = self.get_mytreeiter(treepath[:-1])
-		if parent.row['n_children'] == 1:
+		if parent.n_children == count:
 			treeiter = self.get_iter(parent.treepath) # not mytreeiter !
 			self.emit('row-has-child-toggled', parent.treepath, treeiter)
 
@@ -803,29 +894,48 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 		# no clear cache here - just update row
 		for treepath in self._find_all_pages(row['name']):
 			treeiter = self.get_iter(treepath) # not mytreeiter !
-			self.cache[treepath].row = row # ensure uptodate info
+			self.cache[tuple(treepath)].row = row # ensure uptodate info
 			self.emit('row-changed', treepath, treeiter)
 
+	def on_page_row_delete(self, o, row):
+		self._deleted_paths = list(self._find_all_pages(row['name']))
+
 	def on_page_row_deleted(self, o, row):
+		# Technically "_deleted_paths" should always be a single path
+		# here, else two things changed at once, and Gtk.TreeView cannot
+		# always deal with that.
+
 		self.flush_cache()
-		for treepath in self._find_all_pages(row['name']):
-			if treepath[-1] == 0 and len(treepath) > 1:
-				self._check_parent_has_child_toggled(treepath)
-			self.emit('row-deleted', treepath)
+		if row['name'] == self._MY_ROOT_NAME:
+			self._MY_ROOT_ID = None
+		else:
+			for treepath in self._deleted_paths:
+				self.emit('row-deleted', treepath)
+				if treepath[-1] == 0 and len(treepath) > 1:
+					self._check_parent_has_child_toggled(treepath, 0)
+
+		self._deleted_paths = None
 
 	def n_children_top(self):
-		return self.db.execute(
-			'SELECT COUNT(*) FROM pages WHERE parent=?', (ROOT_ID,)
-		).fetchone()[0]
+		if self._MY_ROOT_ID is None:
+			return 0
+		else:
+			return self.db.execute(
+				'SELECT COUNT(*) FROM pages WHERE parent=?', (self._MY_ROOT_ID,)
+			).fetchone()[0]
 
 	def get_mytreeiter(self, treepath):
+		if self._MY_ROOT_ID is None:
+			return None
+
+		treepath = tuple(treepath) # used to cache
 		if treepath in self.cache:
 			return self.cache[treepath]
 
 		# Find parent
 		parentpath = treepath[:-1]
 		if not parentpath:
-			parent_id = ROOT_ID
+			parent_id = self._MY_ROOT_ID
 		else:
 			parent_iter = self.cache.get(parentpath, None) \
 							or self.get_mytreeiter(parentpath) # recurs
@@ -836,21 +946,39 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 
 		# Now cache a slice at the target level
 		offset = treepath[-1]
-		for i, row in enumerate(self.db.execute('''
-			SELECT * FROM pages WHERE parent=?
-			ORDER BY sortkey, name LIMIT 20 OFFSET ?
-			''',
-			(parent_id, offset)
-		)):
-			mytreepath = parentpath + (offset + i,)
+		if self._REVERSE:
+			rows = self.db.execute('''
+				SELECT * FROM pages WHERE parent=?
+				ORDER BY sortkey DESC, name DESC LIMIT 20 OFFSET ?
+				''',
+				(parent_id, offset)
+			)
+		else:
+			rows = self.db.execute('''
+				SELECT * FROM pages WHERE parent=?
+				ORDER BY sortkey ASC, name ASC LIMIT 20 OFFSET ?
+				''',
+				(parent_id, offset)
+			)
+		for i, row in enumerate(rows):
+			mytreepath = tuple(parentpath) + (offset + i,)
 			if mytreepath not in self.cache:
-				self.cache[mytreepath] = MyTreeIter(mytreepath, row, row['n_children'], IS_PAGE)
+				self.cache[mytreepath] = MyTreeIter(
+					Gtk.TreePath(mytreepath),
+					row,
+					row['n_children'],
+					IS_PAGE
+				)
 			else:
 				break # avoid overwriting cache because of ref count
 
 		return self.cache.get(treepath, None)
 
 	def find(self, path):
+		'''Returns the C{Gtk.TreePath} for a notebook page L{Path}
+		If the L{Path} appears multiple times returns the first occurrence
+		@raises IndexNotFoundError: if path not found
+		'''
 		if path.isroot:
 			raise ValueError
 		treepaths = sorted(self._find_all_pages(path.name))
@@ -860,6 +988,10 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 			raise IndexNotFoundError(path)
 
 	def find_all(self, path):
+		'''Returns a list of C{Gtk.TreePath} for a notebook page L{Path}
+		Returns all occurrences in the treeview
+		@raises IndexNotFoundError: if path not found
+		'''
 		if path.isroot:
 			raise ValueError
 		treepaths = self._find_all_pages(path.name)
@@ -869,12 +1001,16 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 			return treepaths
 
 	def _find_all_pages(self, name, update_cache=True):
-		parent_id = ROOT_ID
-		names = name.split(':')
+		if self._MY_ROOT_ID is None or \
+			not name.startswith(self._MY_ROOT_NAME_C):
+				return []
+
+		parent_id = self._MY_ROOT_ID
+		names = name[len(self._MY_ROOT_NAME_C):].split(':')
 		treepath = []
 		for i, basename in enumerate(names):
 			# Get treepath
-			name = ':'.join(names[:i + 1])
+			name = self._MY_ROOT_NAME_C + ':'.join(names[:i + 1])
 			myrow = self.db.execute(
 				'SELECT * FROM pages WHERE name=?', (name,)
 			).fetchone()
@@ -882,13 +1018,22 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 				raise IndexNotFoundError
 
 			sortkey = myrow['sortkey']
-			row = self.db.execute('''
-				SELECT COUNT(*) FROM pages
-				WHERE parent=? and (
-					sortkey<? or (sortkey=? and name<?)
-				)''',
-				(parent_id, sortkey, sortkey, name)
-			).fetchone()
+			if self._REVERSE:
+				row = self.db.execute('''
+					SELECT COUNT(*) FROM pages
+					WHERE parent=? and (
+						sortkey>? or (sortkey=? and name>?)
+					)''',
+					(parent_id, sortkey, sortkey, name)
+				).fetchone()
+			else:
+				row = self.db.execute('''
+					SELECT COUNT(*) FROM pages
+					WHERE parent=? and (
+						sortkey<? or (sortkey=? and name<?)
+					)''',
+					(parent_id, sortkey, sortkey, name)
+				).fetchone()
 			treepath.append(row[0])
 			parent_id = myrow['id']
 
@@ -896,10 +1041,15 @@ class PagesTreeModelMixin(TreeModelMixinBase):
 				# Update cache (avoid overwriting because of ref count)
 				mytreepath = tuple(treepath)
 				if mytreepath not in self.cache:
-					myiter = MyTreeIter(mytreepath, myrow, myrow['n_children'], IS_PAGE)
+					myiter = MyTreeIter(
+						Gtk.TreePath(mytreepath),
+						myrow,
+						myrow['n_children'],
+						IS_PAGE
+					)
 					self.cache[mytreepath] = myiter
 
-		return [tuple(treepath)]
+		return [Gtk.TreePath(treepath)]
 
 
 ########################################################################
@@ -941,7 +1091,7 @@ class TestPagesDBTable(object):
 				if not row['is_link_placeholder']:
 					# Check upwards - parent(s) must not be placeholder either
 					self.assertFalse(parent['is_link_placeholder'],
-						'Placeholder status for parent of %s is inconcsisten' % row['name']
+						'Placeholder status for parent of %s is inconsistent' % row['name']
 					)
 
 	def assertPagesDBEquals(self, db, pages):
