@@ -1,9 +1,8 @@
-# -*- coding: utf-8 -*-
 
 # Copyright 2009-2012 Jaap Karssenberg <jaap.karssenberg@gmail.com>
 # Copyright 2012 Damien Accorsi <damien.accorsi@free.fr>
 
-from __future__ import with_statement
+
 
 import os
 import logging
@@ -17,8 +16,6 @@ logger = logging.getLogger('zim.vcs.bzr')
 
 class BZRApplicationBackend(VCSApplicationBase):
 
-	use_staging = False
-
 	@classmethod
 	def build_bin_application_instance(cls):
 		return Application(('bzr',))
@@ -39,8 +36,7 @@ class BZRApplicationBackend(VCSApplicationBase):
 		if isinstance(versions, (tuple, list)):
 			assert 1 <= len(versions) <= 2
 			if len(versions) == 2:
-				versions = map(int, versions)
-				versions.sort()
+				versions = sorted(map(int, versions))
 				return ['-r', '%i..%i' % tuple(versions)]
 			else:
 				versions = versions[0]
@@ -65,7 +61,7 @@ class BZRApplicationBackend(VCSApplicationBase):
 			return self.run(['add', path])
 
 
-	def annotate(self, file, version):
+	def annotate(self, file, version=None):
 		"""FIXME Document
 		return
 		1 | line1
@@ -87,14 +83,14 @@ class BZRApplicationBackend(VCSApplicationBase):
 		Runs: bzr commit -m {{MSG}} {{PATH}}
 		"""
 		params = ['commit']
-		if msg!='' and msg!=None:
+		if msg != '' and msg is not None:
 			params.append('-m')
 			params.append(msg)
-		if path!='' and path!=None:
+		if path != '' and path is not None:
 			params.append(path)
 		return self.run(params)
 
-	def diff(self, versions, path=None):
+	def diff(self, versions=None, file=None):
 		"""
 		Runs:
 			bzr diff {{REVISION_ARGS}}
@@ -102,11 +98,11 @@ class BZRApplicationBackend(VCSApplicationBase):
 			bzr diff {{REVISION_ARGS}} {{PATH}}
 		"""
 		revision_args = self.build_revision_arguments(versions)
-		if path==None:
+		if file is None:
 			return self.pipe(['diff'] + revision_args)
 			# Using --git option allow to show the renaming of files
 		else:
-			return self.pipe(['diff', path] + revision_args)
+			return self.pipe(['diff', file] + revision_args)
 
 	def ignore(self, file_to_ignore_regexp):
 		"""
@@ -123,7 +119,7 @@ class BZRApplicationBackend(VCSApplicationBase):
 		self.add('.')
 
 	def repo_exists(self):
-		return self.root.subdir('.bzr').exists()
+		return self.root.folder('.bzr').exists()
 
 	def init(self):
 		"""
@@ -131,6 +127,13 @@ class BZRApplicationBackend(VCSApplicationBase):
 		"""
 		return self.run(['init'])
 
+	def is_modified(self):
+		"""Returns true if the repo is not up-to-date, or False
+		@returns: True if the repo is not up-to-date, or False
+		"""
+		# If status return an empty answer, this means the local repo is up-to-date
+		return ''.join(self.status()).strip() != ''
+
 	def log(self, path=None):
 		"""
 		Runs: bzr log --forward {{PATH}}
@@ -175,7 +178,7 @@ class BZRApplicationBackend(VCSApplicationBase):
 				date = line[11:].strip()
 			elif line.startswith('message:'):
 				seenmsg = True
-				msg = u''
+				msg = ''
 			elif seenmsg and line.startswith('  '):
 				msg += line[2:]
 
@@ -189,7 +192,7 @@ class BZRApplicationBackend(VCSApplicationBase):
 		"""
 		Runs: bzr mv --after {{OLDPATH}} {{NEWPATH}}
 		"""
-		self.run(['add', '--no-recurse', newpath.dir])
+		self.run(['add', '--no-recurse', newpath.dirname])
 		return self.run(['mv', oldpath, newpath])
 
 
@@ -199,7 +202,7 @@ class BZRApplicationBackend(VCSApplicationBase):
 		"""
 		return self.run(['rm', path])
 
-	def revert(self, path, version):
+	def revert(self, path=None, version=None):
 		"""
 		Runs:
 			bzr revert {{PATH}} {{REV_ARGS}}
@@ -229,7 +232,7 @@ class BZRApplicationBackend(VCSApplicationBase):
 		"""
 		try:
 			self.run(['whoami'])
-		except ApplicationError, e:
+		except ApplicationError as e:
 			return False
 		else:
 			return True
